<?php
define('HFDC_CHECK_STRING',		1);
define('HFDC_CHECK_ENGLISH',		2);
define('HFDC_CHECK_EMAIL',		3);
define('HFDC_CHECK_PASSWORD',		4);
define('HFDC_CHECK_NUMBER',		5);
define('HFDC_CHECK_REGEX',		10);

define('HFDC_CHECK_OK',			1);
define('HFDC_ERROR_EMPTY',		-1);
define('HFDC_ERROR_UNMATCH',		-2);
define('HFDC_ERROR_INVALID_PARAM',	-3);
define('HFDC_ERROR_UNKNOWN_TYPE',	-4);
define('HFDC_ERROR_OVER_LENGTH',	-5);

class HFormData {
    var $_vars;

    function HFormData() {
        $this->_vars = array();
    }

    function addVariable($name, $title,
                         $check_type =HFDC_CHECK_STRING,
                         $enable_empty =false) {
        $this->setAttr($name, 'title', $title);
        $this->setAttr($name, 'check_type', $check_type);
        $this->setAttr($name, 'enable_empty', $enable_empty);
    }

    function setAttr($name, $type, $val) {
        $this->_vars[$name][$type] = $val;
    }

    function getAttr($name, $type) {
        $result = null;
        if (isset($this->_vars[$name]) && isset($this->_vars[$name][$type])) {
            $result = $this->_vars[$name][$type];
        }
        return $result;
    }

    function getNames() {
        return array_keys($this->_vars);
    }

    function clearAttr($type) {
        foreach ($this->_vars as $name => $val) {
            if (isset($this->_vars[$name][$type])) {
                unset($this->_vars[$name][$type]);
            }
        }
    }

    function reset() {
        $this->clearAttr('value');
        $this->clearAttr('check_result');
        $this->clearAttr('error_message');
    }

    function setEnableEmpty($name, $enable =true) {
        $this->setAttr($name, 'enable_empty', $enable);
    }
  
    function setLength($name, $formSize, $maxlength = 0) {
        if ($maxlength == 0) {
            $maxlength = $formSize;
        }
        $this->setAttr($name, 'size', $formSize);
        $this->setAttr($name, 'length', $maxlength);
    }

    function getLength($name) {
        $size = $this->getAttr($name, 'size');
        $length = $this->getAttr($name, 'length');
        return array($size, $length);
    }
  
    function getValue($name) {
        return $this->getAttr($name, 'value');
    }

    function setValue($name, $value) {
        $this->setAttr($name, 'value', $value);
    }

    function safe($name) {
        return htmlspecialchars($this->getValue($name));
    }
  
    function getCheck($name) {
        $result = $this->getAttr($name, 'check_result');
        if ($result == HFDC_CHECK_OK) {
            return $result;
        }
        return array($result, $this->getAttr($name, 'error_message'));
    }

    function getTitle($name) {
        return $this->getAttr($name, 'title');
    }
  
    function getErrors() {
        $errors = array();
        $names = $this->getNames();
        foreach ($names as $name) {
            $err = $this->getAttr($name, 'error_message');
            if (!empty($err))
                $errors[$name] = $err;
        }
        if (count($errors) == 0) {
            $errors = null;
        }
        return $errors;
    }

    function addGeneralError($message) {
        $this->setAttr('_general', 'error_message', $message);
    }

    function retrieve($input) {
        switch ($input) {
        case 'post':
            if (isset($_POST)) {
                $vals = &$_POST;
            }
            else {
                $vals = &$GLOBALS['HTTP_POST_VARS'];
            }
            break;
        case 'get':
            if (isset($_GET)) {
                $vals = &$_GET;
            }
            else {
                $vals = &$GLOBALS['HTTP_GET_VARS'];
            }
            break;
        case 'request':
            if (isset($_REQUEST)) {
                $vals = &$_REQUEST;
            }
            else {
                $vals = array_merge($GLOBALS['HTTP_GET_VARS'],
                                    $GLOBALS['HTTP_POST_VARS']);
            }
            break;
        }
        $this->reset();
    
        foreach ($this->_vars as $name => $val) {
            if (isset($vals[$name])) {
                $value = $vals[$name];
                $this->setValue($name, $value);

                $title = $this->getAttr($name, 'title');
                $type = $this->getAttr($name, 'check_type');
                $enable_empty = $this->getAttr($name, 'enable_empty');

                $check = HFormDataChecker::check($value, $type, $enable_empty);
                if ($check != HFDC_CHECK_OK) {
                    $this->setAttr($name, 'error_message',
                                   HFormDataChecker::errorMessage($check, $title));
                }
                else {
                    // max length check
                    list($size, $maxlen) = $this->getLength($name);
                    if (!empty($maxlen)) {
                        $check = HFormDataChecker::checkLength($value, $maxlen);
                        if ($check != HFDC_CHECK_OK) {
                            $this->setAttr($name, 'error_message',
                                           HFormDataChecker::errorMessage($check, $title));
                        }
                    }
                }
                $this->setAttr($name, 'check_result', $check);
            }
        }
    }

}

class HFormDataChecker {
    /*
     *  check($string, $type)
     *  check($string, $type, $enable_empty)
     *  check($string, $type, $regex, $enable_empty)
     */
    function check($string, $type) {
        $nargs = func_num_args();
        $args = func_get_args();

        if ($nargs <= 2)
            $enable_empty = false;
        else
            $enable_empty = $args[2];

        switch ($type) {
        case HFDC_CHECK_STRING:
            $result = HFDC_CHECK_OK;
            break;
        case HFDC_CHECK_ENGLISH:
            $regex = '/^[A-Za-z0-9_.+-\s]+$/';
            break;
        case HFDC_CHECK_PASSWORD:
            $regex = '/^[\040-\176]+$/';
            break;
        case HFDC_CHECK_EMAIL:
            $regex = '/^[A-Za-z0-9_.+-]+@[A-Za-z0-9.-]+$/';
            break;
        case HFDC_CHECK_NUMBER:
            $regex = '/^[0-9]+$/';
            break;
        case HFDC_CHECK_REGEX:
            $regex = $args[2];
            if ($nargs == 4)
                $enable_empty = $args[3];

            if (empty($regex) || is_string($regex)) {
                $result = HFDC_ERROR_INVLID_PARAM;
            }
            break;
        default:
            $result = HFDC_ERROR_UNKNOWN_TYPE;
            break;
        }

        if (empty($string)) {
            if ($enable_empty)
                $result = HFDC_CHECK_OK;
            else
                $result = HFDC_ERROR_EMPTY;
        }

        if (!isset($result) && isset($regex)) {
            if (preg_match($regex, $string))
                $result = HFDC_CHECK_OK;
            else
                $result = HFDC_ERROR_UNMATCH;
        }
        return $result;
    }

    function checkLength($string, $length) {
        $result = HFDC_CHECK_OK;
        if (strlen($string) > $length) {
            $result = HFDC_ERROR_OVER_LENGTH;
        }
        return $result;
    }
  
    function errorMessage($errcode, $prefix = '') {
        static $messages =
            array(HFDC_ERROR_EMPTY          => 'ϤƤޤ',
                  HFDC_ERROR_UNMATCH        => 'ηޤ',
                  HFDC_ERROR_INVALID_PARAM  => '(顼)',
                  HFDC_ERROR_UNKNOWN_TYPE   => '(顼)',
                  HFDC_ERROR_OVER_LENGTH    => 'ʸĶƤޤ',
                  );
        if (!isset($messages[$errcode]))
            $mesg = '顼ɤְäƤޤ';
        else
            $mesg = $prefix . $messages[$errcode];
        return $mesg;
    }
}
?>